'use strict';

var Keywalk = require('keywalk');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Keywalk__default = /*#__PURE__*/_interopDefaultLegacy(Keywalk);

var CSS_CLASS_PREFIX = 'inpd';
var DROPDOWN_CONTAINER_CLASS = 'dropdown-container';
var DROPDOWN_ITEM_CLASS = 'dropdown-item';
var DROPDOWN_ITEM_ICON_CLASS = 'dropdown-item__icon';
var DROPDOWN_ITEM_TITLE_CLASS = 'dropdown-item__title';
var DROPDOWN_ITEM_SUBTITLE_CLASS = 'dropdown-item__subtitle';

var InputDropdown = /** @class */ (function () {
    function InputDropdown(args) {
        var _this = this;
        var selector = args.selector, data = args.data, dataAliases = args.dataAliases, dropdownWidth = args.dropdownWidth, dropdownClass = args.dropdownClass, _a = args.useKeywalk, useKeywalk = _a === void 0 ? false : _a;
        this.args = args;
        this.selector = selector;
        this.dropdownWidth = dropdownWidth;
        this.dropdownClass = dropdownClass;
        this.data = data;
        this.dataAliases = dataAliases;
        this.useKeywalk = useKeywalk;
        this.isDropdownInserted = false;
        this.dropdownSpecificClass = "inpd-dropdown-container-" + Math.random()
            .toString(36)
            .substr(2, 5);
        this.initInput();
        this.initDropdown();
        document.addEventListener('click', function (ev) {
            _this.detectOnFocusInput(ev.target);
            _this.detectOnBlurInputOrDropdown(ev.target);
        });
    }
    InputDropdown.prototype.initInput = function () {
        var _this = this;
        this.input = document.querySelector(this.selector);
        // get the position after window loaded to avoid layout shifting
        window.addEventListener('load', function () {
            _this.setInputPosition();
        });
        window.addEventListener('resize', function () {
            _this.setInputPosition();
        });
        this.observeDOM(document.documentElement, function () {
            _this.setInputPosition();
        });
        this.input.addEventListener('input', function (e) {
            _this.showDropdown();
            if (e.target.value.length === 0)
                _this.hideDropdown();
        });
    };
    InputDropdown.prototype.setInputPosition = function () {
        var _a = this.input.getBoundingClientRect(), height = _a.height, width = _a.width, left = _a.left, top = _a.top;
        this.inputHeight = height;
        this.inputWidth = width;
        this.inputLeft = left;
        this.inputTop = top;
        if (this.dropdown)
            this.setDropdownPosition();
    };
    InputDropdown.prototype.initDropdown = function () {
        this.dropdown = document.createElement('div');
        this.dropdown.classList.add(CSS_CLASS_PREFIX + "-" + DROPDOWN_CONTAINER_CLASS);
        this.dropdown.classList.add(this.dropdownSpecificClass);
        if (this.dropdownClass)
            this.dropdown.classList.add(this.dropdownClass);
        this.setDropdownPosition();
    };
    InputDropdown.prototype.setDropdownPosition = function () {
        this.dropdown.style.width = !this.dropdownWidth
            ? this.inputWidth + "px"
            : this.dropdownWidth;
        this.dropdown.style.left = this.inputLeft + "px"; // TODO: add center option
        this.dropdown.style.top = this.inputTop + this.inputHeight + "px"; // TODO: add distance
    };
    InputDropdown.prototype.setDropdownItems = function () {
        var _this = this;
        var _a;
        this.dropdown.innerHTML = '';
        (_a = this.data) === null || _a === void 0 ? void 0 : _a.forEach(function (obj) {
            var item = document.createElement('div');
            item.classList.add(CSS_CLASS_PREFIX + "-" + DROPDOWN_ITEM_CLASS);
            if (_this.dataAliases.icon) {
                var itemIcon = document.createElement('div');
                itemIcon.classList.add(CSS_CLASS_PREFIX + "-" + DROPDOWN_ITEM_ICON_CLASS);
                itemIcon.style.backgroundImage = "url(" + obj[_this.dataAliases.icon] + ")";
                item.appendChild(itemIcon);
            }
            if (_this.dataAliases.title) {
                var itemTitle = document.createElement('div');
                itemTitle.classList.add(CSS_CLASS_PREFIX + "-" + DROPDOWN_ITEM_TITLE_CLASS);
                itemTitle.innerHTML = obj[_this.dataAliases.title];
                item.appendChild(itemTitle);
            }
            if (_this.dataAliases.subtitle) {
                var itemSubtitle = document.createElement('div');
                itemSubtitle.classList.add(CSS_CLASS_PREFIX + "-" + DROPDOWN_ITEM_SUBTITLE_CLASS);
                itemSubtitle.innerHTML = obj[_this.dataAliases.subtitle];
                item.appendChild(itemSubtitle);
            }
            _this.dropdown.appendChild(item);
        });
        if (!this.isDropdownInserted)
            this.insertDropdown();
    };
    InputDropdown.prototype.insertDropdown = function () {
        document.body.appendChild(this.dropdown);
        this.isDropdownInserted = true;
        if (this.useKeywalk)
            this.initKeywalk();
    };
    InputDropdown.prototype.showDropdown = function () {
        this.dropdown.style.display = 'block';
    };
    InputDropdown.prototype.hideDropdown = function () {
        this.dropdown.style.display = 'none';
        if (this.keywalkInstance)
            this.keywalkInstance.reset();
    };
    InputDropdown.prototype.detectOnFocusInput = function (el) {
        if (this.input.value.length > 0 &&
            (el.matches(this.selector) ||
                el.matches(this.selector + " *") ||
                el.matches("." + this.dropdownSpecificClass) ||
                el.matches("." + this.dropdownSpecificClass + " *"))) {
            this.showDropdown();
        }
    };
    InputDropdown.prototype.detectOnBlurInputOrDropdown = function (el) {
        if (!el.matches(this.selector) &&
            !el.matches(this.selector + " *") &&
            !el.matches("." + this.dropdownSpecificClass) &&
            !el.matches("." + this.dropdownSpecificClass + " *")) {
            this.hideDropdown();
        }
    };
    InputDropdown.prototype.initKeywalk = function () {
        var _this = this;
        this.keywalkInstance = new Keywalk__default['default']({
            trigger: this.input,
            container: this.dropdown,
            onWalk: function (element, index) {
                _this.emitOnWalk(element, index);
            },
            onSelect: function (element, index) {
                _this.emitOnSelect(element, index);
            }
        });
    };
    InputDropdown.prototype.emitOnWalk = function (element, index) {
        if (this.args.onWalk)
            this.args.onWalk(element, index, this.data[index]);
    };
    InputDropdown.prototype.emitOnSelect = function (element, index) {
        if (this.args.onSelect)
            this.args.onSelect(element, index, this.data[index]);
    };
    InputDropdown.prototype.updateData = function (updatedData) {
        this.data = updatedData;
        this.setDropdownItems();
    };
    InputDropdown.prototype.observeDOM = function (obj, callback) {
        var MutationObserver = window.MutationObserver || window.WebKitMutationObserver;
        if (!obj || obj.nodeType !== 1)
            return;
        if (MutationObserver) {
            // define a new observer
            var mutationObserver = new MutationObserver(callback);
            // have the observer observe foo for changes in children
            mutationObserver.observe(obj, {
                attributes: true,
                childList: true,
                subtree: true
            });
            return mutationObserver;
        }
        // browser support fallback
        else if (window.addEventListener) {
            obj.addEventListener('DOMNodeInserted', callback, false);
            obj.addEventListener('DOMNodeRemoved', callback, false);
        }
    };
    return InputDropdown;
}());

module.exports = InputDropdown;
